﻿using UnityEngine;
using System.Collections.Generic;
using SimpleJSON;

namespace MacGruber
{
    public class Counter : MVRScript
    {
		private JSONStorableFloat myCount;
		private JSONStorableFloat myValue;
		private EventTrigger myUpTrigger;
		private EventTrigger myDownTrigger;
		private FloatTrigger myValueTrigger;
		

        public override void Init()
		{
			Utils.SetupInfoText(this,
				"<color=#606060><size=40><b>Counter</b></size>\nLogicBrick that can count up and down and provides a status value between 0 and 1. Provides a trigger signal when reaching either end of the range.</color>",
				200.0f, true
			);

			myCount = Utils.SetupSliderInt(this, "Count", 3, 1, 50, false);
			
			myValue = new JSONStorableFloat("Value", 0, 0, 50, true, true);
			myValue.storeType = JSONStorableParam.StoreType.Full;
			RegisterFloat(myValue);

			myUpTrigger = new EventTrigger(this, "OnUpTrigger");
			myDownTrigger = new EventTrigger(this, "OnDownTrigger");
			myValueTrigger = new FloatTrigger(this, "OnChangeTrigger");
			Utils.SetupButton(this, "Actions OnUpTrigger", myUpTrigger.OpenPanel, false);
			Utils.SetupButton(this, "Actions OnDownTrigger", myDownTrigger.OpenPanel, false);
			Utils.SetupButton(this, "Actions OnChangeTrigger", myValueTrigger.OpenPanel, false);

			Utils.SetupButton(this, "Test SetUP", SetUp, false);
			Utils.SetupAction(this, "SetUP", SetUp);
			Utils.SetupButton(this, "Test SetDOWN", SetDown, false);
			Utils.SetupAction(this, "SetDOWN", SetDown);
			Utils.SetupButton(this, "Test CountUP", CountUp, false);
			Utils.SetupAction(this, "CountUP", CountUp);
			Utils.SetupButton(this, "Test CountDOWN", CountDown, false);
			Utils.SetupAction(this, "CountDOWN", CountDown);

			SuperController.singleton.onAtomUIDRenameHandlers += OnAtomRename;
			SimpleTriggerHandler.LoadAssets();
		}

		private void OnAtomRename(string oldid, string newid)
		{
			myUpTrigger.SyncAtomNames();
			myDownTrigger.SyncAtomNames();
			myValueTrigger.SyncAtomNames();
		}

		public override JSONClass GetJSON(bool includePhysical = true, bool includeAppearance = true, bool forceStore = false)
		{
			JSONClass jc = base.GetJSON(includePhysical, includeAppearance, forceStore);
			if (includePhysical || forceStore)
			{
				needsStore = true;
				jc[myUpTrigger.Name] = myUpTrigger.GetJSON(base.subScenePrefix);
				jc[myDownTrigger.Name] = myDownTrigger.GetJSON(base.subScenePrefix);
				jc[myValueTrigger.Name] = myValueTrigger.GetJSON(base.subScenePrefix);
			}
			return jc;
		}

		public override void LateRestoreFromJSON(JSONClass jc, bool restorePhysical = true, bool restoreAppearance = true, bool setMissingToDefault = true)
		{
			base.LateRestoreFromJSON(jc, restorePhysical, restoreAppearance, setMissingToDefault);
			if (!base.physicalLocked && restorePhysical && !IsCustomPhysicalParamLocked("trigger"))
			{
				myUpTrigger.RestoreFromJSON(jc, base.subScenePrefix, base.mergeRestore, setMissingToDefault);
				myDownTrigger.RestoreFromJSON(jc, base.subScenePrefix, base.mergeRestore, setMissingToDefault);
				myValueTrigger.RestoreFromJSON(jc, base.subScenePrefix, base.mergeRestore, setMissingToDefault);
			}
		}

		private void OnDestroy()
		{
			SuperController.singleton.onAtomUIDRenameHandlers -= OnAtomRename;
			myUpTrigger.Remove();
			myDownTrigger.Remove();
			myValueTrigger.Remove();
		}
		
		private void Update()
		{
			myUpTrigger.Update();
			myDownTrigger.Update();
			myValueTrigger.Update();
		}
		
		private void CountUp()
		{
			int val = Mathf.RoundToInt(myValue.val);			
			int count = Mathf.RoundToInt(myCount.val);
			myValue.val = val = Mathf.Min(val+1, count);
			
			myValueTrigger.Trigger(val / myCount.val);
			if (val >= myCount.val)
				myUpTrigger.Trigger();
		}
		
		private void CountDown()
		{
			int val = Mathf.RoundToInt(myValue.val);
			myValue.val = val = Mathf.Max(val-1, 0);
			
			myValueTrigger.Trigger(val / myCount.val);
			if (val <= 0)
				myDownTrigger.Trigger();
		}
		
		private void SetUp()
		{
			myValue.val = Mathf.Round(myCount.val);
			myValueTrigger.Trigger(1.0f);
		}

		private void SetDown()
		{
			myValue.val = 0.0f;
			myValueTrigger.Trigger(0.0f);
		}
    }
}
