﻿using UnityEngine;
using System.Collections.Generic;
using SimpleJSON;

namespace MacGruber
{
    public class EventAudioQueueEmpty : MVRScript
    {
		private EventTrigger myEventTrigger;
		private AudioSourceControl myAudioSource;
		private bool myTriggered = false;
		private float myClock = 0.0f;

        public override void Init()
		{
			string storableName = string.Empty;
			if (containingAtom.type == "Person")
				storableName = "HeadAudioSource";
			else if (containingAtom.type == "AudioSource")
				storableName = "AudioSource";
			else if (containingAtom.type == "AptSpeaker")
				storableName = "AptSpeaker_Import";
			else if (containingAtom.type == "RhythmAudioSource")
				storableName = "RhythmSource";
			myAudioSource = containingAtom.GetStorableByID(storableName) as AudioSourceControl;
			
			if (myAudioSource == null)
			{
				SuperController.LogError("The EventAudioQueueEmpty plugin needs to be placed on a Person, AudioSource, RhythmAudioSource or AptSpeaker atom.");
				Utils.SetupInfoText(this,
					"<color=#FF6060><size=40><b>EventAudioQueueEmpty</b></size>\nThis plugin needs to be placed on a Person, AudioSource, RhythmAudioSource or AptSpeaker atom.</color>",
					500.0f, true
				);
				return;
			}
			
			Utils.SetupInfoText(this,
				"<color=#606060><size=40><b>EventAudioQueueEmpty</b></size>\nLogicBrick that triggers when the queue of the <i>AudioSource</i> it is placed on is empty.</color>",
				200.0f, true
			);

			myEventTrigger = new EventTrigger(this, "OnTrigger");
			Utils.SetupButton(this, "Actions OnTrigger", myEventTrigger.OpenPanel, false);

			Utils.SetupButton(this, "Test Trigger", Trigger, false);

			SuperController.singleton.onAtomUIDRenameHandlers += OnAtomRename;
			SimpleTriggerHandler.LoadAssets();
		}

		private void OnAtomRename(string oldid, string newid)
		{
			myEventTrigger.SyncAtomNames();
		}

		public override JSONClass GetJSON(bool includePhysical = true, bool includeAppearance = true, bool forceStore = false)
		{
			JSONClass jc = base.GetJSON(includePhysical, includeAppearance, forceStore);
			if (includePhysical || forceStore)
			{
				needsStore = true;
				jc[myEventTrigger.Name] = myEventTrigger.GetJSON(base.subScenePrefix);
			}
			return jc;
		}

		public override void LateRestoreFromJSON(JSONClass jc, bool restorePhysical = true, bool restoreAppearance = true, bool setMissingToDefault = true)
		{
			base.LateRestoreFromJSON(jc, restorePhysical, restoreAppearance, setMissingToDefault);
			if (!base.physicalLocked && restorePhysical && !IsCustomPhysicalParamLocked("trigger"))
			{
				myEventTrigger.RestoreFromJSON(jc, base.subScenePrefix, base.mergeRestore, setMissingToDefault);
			}
		}

		private void OnDestroy()
		{
			SuperController.singleton.onAtomUIDRenameHandlers -= OnAtomRename;
			myEventTrigger.Remove();
		}
		
		private void Update()
		{
			myEventTrigger.Update();
			
			if (myAudioSource == null)
				return;
			
			if (myAudioSource.playingClip != null)
			{
				myTriggered = false;
				myClock	= 0.0f;
			}
			else if (!myTriggered)
			{
				myClock += Time.unscaledDeltaTime;
				if (myClock > myAudioSource.delayBetweenQueuedClips)
					Trigger();
			}
		}

		private void Trigger()
		{
			myTriggered = true;
			myEventTrigger.Trigger();
		}
    }
}
