﻿using UnityEngine;
using UnityEngine.XR;
using System.Collections.Generic;
using SimpleJSON;

namespace MacGruber
{
    public class EventSceneLoad : MVRScript
    {
		private JSONStorableBool myTriggerVR;
		private JSONStorableBool myTriggerDesktop;
		private EventTrigger myEventTrigger;
		private bool myQueueTrigger = false;

        public override void Init()
		{			
			Utils.SetupInfoText(this,
				"<color=#606060><size=40><b>EventSceneLoad</b></size>\nLogicBrick that triggers after a VaM scene finished loading.</color>",
				200.0f, true
			);
			
			myTriggerVR = Utils.SetupToggle(this, "Trigger in VR-Mode", true, false);
			myTriggerDesktop = Utils.SetupToggle(this, "Trigger in Desktop-Mode", true, false);

			myEventTrigger = new EventTrigger(this, "OnTrigger");
			Utils.SetupButton(this, "Actions OnTrigger", myEventTrigger.OpenPanel, false);

			Utils.SetupButton(this, "Test Trigger", Trigger, false);

			SuperController.singleton.onAtomUIDRenameHandlers += OnAtomRename;
			SuperController.singleton.onSceneLoadedHandlers += OnSceneLoaded;
			SimpleTriggerHandler.LoadAssets();
		}

		private void OnAtomRename(string oldid, string newid)
		{
			myEventTrigger.SyncAtomNames();
		}

		public override JSONClass GetJSON(bool includePhysical = true, bool includeAppearance = true, bool forceStore = false)
		{
			JSONClass jc = base.GetJSON(includePhysical, includeAppearance, forceStore);
			if (includePhysical || forceStore)
			{
				needsStore = true;
				jc[myEventTrigger.Name] = myEventTrigger.GetJSON(base.subScenePrefix);
			}
			return jc;
		}

		public override void LateRestoreFromJSON(JSONClass jc, bool restorePhysical = true, bool restoreAppearance = true, bool setMissingToDefault = true)
		{
			base.LateRestoreFromJSON(jc, restorePhysical, restoreAppearance, setMissingToDefault);
			if (!base.physicalLocked && restorePhysical && !IsCustomPhysicalParamLocked("trigger"))
			{
				myEventTrigger.RestoreFromJSON(jc, base.subScenePrefix, base.mergeRestore, setMissingToDefault);
			}

			if (!SuperController.singleton.isLoading)
				myQueueTrigger = true; // Work around OnSceneLoaded already being triggered before the plugin runs in case it was not yet allowed to run by the user.
		}

		private void OnDestroy()
		{
			SuperController.singleton.onSceneLoadedHandlers -= OnSceneLoaded;
			SuperController.singleton.onAtomUIDRenameHandlers -= OnAtomRename;
			myEventTrigger.Remove();
		}
		
		private void OnSceneLoaded()
		{
			myQueueTrigger = true;
		}
		
		private void Update()
		{
			myEventTrigger.Update();
			
			if (myQueueTrigger)
				Trigger();
		}

		private void Trigger()
		{		
			myQueueTrigger = false;
			
			bool isDesktopMode = !XRSettings.isDeviceActive || string.IsNullOrEmpty(XRSettings.loadedDeviceName);
			if (isDesktopMode && myTriggerDesktop.val)
				myEventTrigger.Trigger();
			else if (!isDesktopMode && myTriggerVR.val)
				myEventTrigger.Trigger();
		}
    }
}
