/* /////////////////////////////////////////////////////////////////////////////////////////////////
LinearAnimation v1 by MacGruber.
Simple plugin that allows a linear animation between two points for the main control of an atom.

Version 1 - 2020-12-26
	Initial release.
	
///////////////////////////////////////////////////////////////////////////////////////////////// */

using UnityEngine;
using System.Collections.Generic;
using SimpleJSON;

namespace MacGruber
{
	public class LinearAnimation : MVRScript
	{
		private JSONStorableFloat myTransition;
		
		private Vector3 myPositionA;
		private Vector3 myPositionB;
		private Quaternion myRotationA;
		private Quaternion myRotationB;
		
		private bool myLoading = false;

		public override void Init()
		{
			SetPositionA();
			SetPositionB();
			
			myTransition = Utils.SetupSliderFloat(this, "Transition", 0.0f, 0.0f, 1.0f, false);
			myTransition.setCallbackFunction += SetTransition;
			
			Utils.SetupButton(this, "Set Position 0", SetPositionA, false);
			Utils.SetupButton(this, "Set Position 1", SetPositionB, false);			
			
			Utils.SetupInfoText(this,
				"<color=#606060><size=40><b>LinearAnimation v0.1</b></size>\nSimple plugin that allows a linear animation between two points for the main control of an atom.</color>",
				200.0f, true
			);
		}
		
		private void SetPositionA()
		{
			Transform mainController = GetMainController().transform;
			myPositionA = mainController.position;
			myRotationA = mainController.rotation;
		}
		
		private void SetPositionB()
		{
			Transform mainController = GetMainController().transform;
			myPositionB = mainController.position;
			myRotationB = mainController.rotation;
		}
		
		private void SetTransition(float t)
		{
			if (myLoading || !enabledJSON.val)
				return;
			
			Vector3 pos = Vector3.Lerp(myPositionA, myPositionB, t);
			Quaternion rot =  Quaternion.Slerp(myRotationA, myRotationB, t);			
			GetMainController().transform.SetPositionAndRotation(pos, rot);
		}

		public override JSONClass GetJSON(bool includePhysical = true, bool includeAppearance = true, bool forceStore = false)
		{
			JSONClass jc = base.GetJSON(includePhysical, includeAppearance, forceStore);
			if (includePhysical || forceStore)
			{
				needsStore = true;
				{
					JSONClass a = new JSONClass();
					JSONClass pos = new JSONClass();
					pos["x"].AsFloat = myPositionA.x;
					pos["y"].AsFloat = myPositionA.y;
					pos["z"].AsFloat = myPositionA.z;
					a["position"] = pos;					
					JSONClass rot = new JSONClass();
					Vector3 euler = myRotationA.eulerAngles;
					rot["x"].AsFloat = euler.x;
					rot["y"].AsFloat = euler.y;
					rot["z"].AsFloat = euler.z;
					a["rotation"] = rot;
					jc["positionA"] = a;
				}
				{
					JSONClass b = new JSONClass();
					JSONClass pos = new JSONClass();
					pos["x"].AsFloat = myPositionB.x;
					pos["y"].AsFloat = myPositionB.y;
					pos["z"].AsFloat = myPositionB.z;
					b["position"] = pos;	
					JSONClass rot = new JSONClass();
					Vector3 euler = myRotationB.eulerAngles;
					rot["x"].AsFloat = euler.x;
					rot["y"].AsFloat = euler.y;
					rot["z"].AsFloat = euler.z;
					b["rotation"] = rot;
					jc["positionB"] = b;
				}
			}
			return jc;
		}
		
		public override void RestoreFromJSON(JSONClass jc, bool restorePhysical = true, bool restoreAppearance = true, JSONArray presetAtoms = null, bool setMissingToDefault = true)
		{
			myLoading = true;
			base.RestoreFromJSON(jc, restorePhysical, restoreAppearance, presetAtoms, setMissingToDefault);
			
			if (!physicalLocked && restorePhysical)
			{
				if (jc.HasKey("positionA"))
				{
					JSONClass a = jc["positionA"].AsObject;				
					if (a.HasKey("position"))
					{
						JSONClass pos = a["position"].AsObject;
						myPositionA.x = pos["x"].AsFloat;
						myPositionA.y = pos["y"].AsFloat;
						myPositionA.z = pos["z"].AsFloat;
					}
					if (a.HasKey("rotation"))
					{
						JSONClass rot = a["rotation"].AsObject;
						Vector3 euler;
						euler.x = rot["x"].AsFloat;
						euler.y = rot["y"].AsFloat;
						euler.z = rot["z"].AsFloat;
						myRotationA.eulerAngles = euler;
					}
				}
				if (jc.HasKey("positionB"))
				{
					JSONClass b = jc["positionB"].AsObject;				
					if (b.HasKey("position"))
					{
						JSONClass pos = b["position"].AsObject;
						myPositionB.x = pos["x"].AsFloat;
						myPositionB.y = pos["y"].AsFloat;
						myPositionB.z = pos["z"].AsFloat;
					}
					if (b.HasKey("rotation"))
					{
						JSONClass rot = b["rotation"].AsObject;
						Vector3 euler;
						euler.x = rot["x"].AsFloat;
						euler.y = rot["y"].AsFloat;
						euler.z = rot["z"].AsFloat;
						myRotationB.eulerAngles = euler;
					}
				}
			}
			
			myLoading = false;
		}
	}
}