﻿using UnityEngine;
using System.Collections.Generic;
using SimpleJSON;

namespace MacGruber
{
    public class Loop : MVRScript
    {
		private JSONStorableBool myAcceptTrigger;
		private JSONStorableFloat myCount;
		private JSONStorableFloat myTime;
		private EventTrigger myEventTrigger;

		private float myClock = -1.0f;
		private int myIndex = 0;

        public override void Init()
		{
			Utils.SetupInfoText(this,
				"<color=#606060><size=40><b>Loop</b></size>\nLogicBrick that triggers multiple times with at a fixed time interval after <i>Trigger</i> is called.</color>",
				150.0f, true
			);
			
			myAcceptTrigger = Utils.SetupToggle(this, "AcceptTrigger", true, false);

			myCount = Utils.SetupSliderInt(this, "Count", 3, 1, 50, false);

			myTime = Utils.SetupSliderFloatWithRange(this, "Time (sec)", 3, 0, 10, false);
			myTime.constrained = false;

			myEventTrigger = new EventTrigger(this, "OnTrigger");
			Utils.SetupButton(this, "Actions OnTrigger", myEventTrigger.OpenPanel, false);

			Utils.SetupButton(this, "Test Trigger", Trigger, false);
			Utils.SetupAction(this, "Trigger", Trigger);

			SuperController.singleton.onAtomUIDRenameHandlers += OnAtomRename;
			SimpleTriggerHandler.LoadAssets();
		}

		private void OnAtomRename(string oldid, string newid)
		{
			myEventTrigger.SyncAtomNames();
		}

		public override JSONClass GetJSON(bool includePhysical = true, bool includeAppearance = true, bool forceStore = false)
		{
			JSONClass jc = base.GetJSON(includePhysical, includeAppearance, forceStore);
			if (includePhysical || forceStore)
			{
				needsStore = true;
				jc[myEventTrigger.Name] = myEventTrigger.GetJSON(base.subScenePrefix);
			}
			return jc;
		}

		public override void LateRestoreFromJSON(JSONClass jc, bool restorePhysical = true, bool restoreAppearance = true, bool setMissingToDefault = true)
		{
			base.LateRestoreFromJSON(jc, restorePhysical, restoreAppearance, setMissingToDefault);
			Utils.AdjustSliderRange(myTime);
			myClock = -1.0f;

			if (!base.physicalLocked && restorePhysical && !IsCustomPhysicalParamLocked("trigger"))
			{
				myEventTrigger.RestoreFromJSON(jc, base.subScenePrefix, base.mergeRestore, setMissingToDefault);
			}
		}

		private void OnDestroy()
		{
			SuperController.singleton.onAtomUIDRenameHandlers -= OnAtomRename;
			myEventTrigger.Remove();
		}

		private void Update()
		{
			myEventTrigger.Update();
			
			if (myClock < 0.0f || myIndex >= myCount.val)
				return;

			myClock -= Time.deltaTime;
			if (myClock <= 0.0f)
			{
				myEventTrigger.Trigger();
				myClock = myTime.val;
				++myIndex;
			}
		}

		private void Trigger()
		{
			if (myAcceptTrigger.val)
			{
				myClock = myTime.val;
				myIndex = 0;
			}
		}
    }
}
