﻿using UnityEngine;
using System.Collections.Generic;
using SimpleJSON;

namespace MacGruber
{
    public class RandomTimer : MVRScript
    {
		private JSONStorableFloat myMinTime;
		private JSONStorableFloat myMaxTime;
		private EventTrigger myEventTrigger;

		private float myClock = -1.0f;

        public override void Init()
		{
			Utils.SetupInfoText(this,
				"<color=#606060><size=40><b>RandomTimer</b></size>\nLogicBrick that triggers continuously in a random time interval.</color>",
				150.0f, true
			);

			myMinTime = Utils.SetupSliderFloatWithRange(this, "Min Time (sec)", 3, 0, 10, false);
			myMinTime.constrained = false;
			myMinTime.setCallbackFunction += (float v) => {
				Utils.AdjustMaxSliderFromMin(v, myMaxTime);
			};

			myMaxTime = Utils.SetupSliderFloatWithRange(this, "Max Time (sec)", 5, 0, 10, false);
			myMaxTime.constrained = false;
			myMaxTime.setCallbackFunction += (float v) => {
				myMinTime.valNoCallback = Mathf.Min(v, myMinTime.val);
			};

			myClock = UnityEngine.Random.Range(myMinTime.val, myMaxTime.val);

			myEventTrigger = new EventTrigger(this, "OnTrigger");
			Utils.SetupButton(this, "Actions OnTrigger", myEventTrigger.OpenPanel, false);

			SuperController.singleton.onAtomUIDRenameHandlers += OnAtomRename;
			SimpleTriggerHandler.LoadAssets();
		}

		private void OnAtomRename(string oldid, string newid)
		{
			myEventTrigger.SyncAtomNames();
		}

		public override JSONClass GetJSON(bool includePhysical = true, bool includeAppearance = true, bool forceStore = false)
		{
			JSONClass jc = base.GetJSON(includePhysical, includeAppearance, forceStore);
			if (includePhysical || forceStore)
			{
				needsStore = true;
				jc[myEventTrigger.Name] = myEventTrigger.GetJSON(base.subScenePrefix);
			}
			return jc;
		}

		public override void LateRestoreFromJSON(JSONClass jc, bool restorePhysical = true, bool restoreAppearance = true, bool setMissingToDefault = true)
		{
			base.LateRestoreFromJSON(jc, restorePhysical, restoreAppearance, setMissingToDefault);
			Utils.AdjustSliderRange(myMinTime);
			Utils.AdjustSliderRange(myMaxTime);
			myClock = UnityEngine.Random.Range(myMinTime.val, myMaxTime.val);

			if (!base.physicalLocked && restorePhysical && !IsCustomPhysicalParamLocked("trigger"))
			{
				myEventTrigger.RestoreFromJSON(jc, base.subScenePrefix, base.mergeRestore, setMissingToDefault);
			}
		}

		private void OnDestroy()
		{
			SuperController.singleton.onAtomUIDRenameHandlers -= OnAtomRename;
			myEventTrigger.Remove();
		}

		private void Update()
		{
			myEventTrigger.Update();
			
			myClock -= Time.deltaTime;
			if (myClock <= 0.0f)
			{
				myEventTrigger.Trigger();
				myClock += UnityEngine.Random.Range(myMinTime.val, myMaxTime.val);
				myClock = Mathf.Max(myClock, 0.0f);
			}
		}
    }
}
