﻿using UnityEngine;
using System.Collections.Generic;
using SimpleJSON;

namespace MacGruber
{
    public class RelayOnce : MVRScript
    {
		private JSONStorableBool myAcceptTrigger;
		private EventTrigger myEventTrigger;
		private bool myTriggered = false;

        public override void Init()
		{
			Utils.SetupInfoText(this,
				"<color=#606060><size=40><b>RelayOnce</b></size>\nLogicBrick that triggers directly after <i>Trigger</i> is called, but only once. Call <i>Reset</i> to use again.</color>",
				200.0f, true
			);

			myTriggered = false;
			
			myAcceptTrigger = Utils.SetupToggle(this, "AcceptTrigger", true, false);

			myEventTrigger = new EventTrigger(this, "OnTrigger");
			Utils.SetupButton(this, "Actions OnTrigger", myEventTrigger.OpenPanel, false);

			Utils.SetupButton(this, "Test Reset", Reset, false);
			Utils.SetupAction(this, "Reset", Reset);
			Utils.SetupButton(this, "Test Trigger", Trigger, false);
			Utils.SetupAction(this, "Trigger", Trigger);

			SuperController.singleton.onAtomUIDRenameHandlers += OnAtomRename;
			SimpleTriggerHandler.LoadAssets();
		}

		private void OnAtomRename(string oldid, string newid)
		{
			myEventTrigger.SyncAtomNames();
		}

		public override JSONClass GetJSON(bool includePhysical = true, bool includeAppearance = true, bool forceStore = false)
		{
			JSONClass jc = base.GetJSON(includePhysical, includeAppearance, forceStore);
			if (includePhysical || forceStore)
			{
				needsStore = true;
				jc[myEventTrigger.Name] = myEventTrigger.GetJSON(base.subScenePrefix);
			}
			return jc;
		}

		public override void LateRestoreFromJSON(JSONClass jc, bool restorePhysical = true, bool restoreAppearance = true, bool setMissingToDefault = true)
		{
			base.LateRestoreFromJSON(jc, restorePhysical, restoreAppearance, setMissingToDefault);
			if (!base.physicalLocked && restorePhysical && !IsCustomPhysicalParamLocked("trigger"))
			{
				myEventTrigger.RestoreFromJSON(jc, base.subScenePrefix, base.mergeRestore, setMissingToDefault);
			}
		}

		private void OnDestroy()
		{
			SuperController.singleton.onAtomUIDRenameHandlers -= OnAtomRename;
			myEventTrigger.Remove();
		}
		
		private void Update()
		{
			myEventTrigger.Update();
		}

		private void Trigger()
		{			
			if (!myAcceptTrigger.val || myTriggered)
				return;
			myEventTrigger.Trigger();
			myTriggered = true;
		}

		private void Reset()
		{
			if (myAcceptTrigger.val)
				myTriggered = false;
		}
    }
}
