﻿using UnityEngine;
using System.Collections.Generic;
using SimpleJSON;

namespace MacGruber
{
    public class Timer : MVRScript
    {
		private JSONStorableFloat myTime;
		private EventTrigger myEventTrigger;

		private float myClock = -1.0f;

        public override void Init()
		{
			Utils.SetupInfoText(this,
				"<color=#606060><size=40><b>Timer</b></size>\nLogicBrick that triggers continuously in a fixed time interval.</color>",
				150.0f, true
			);

			myEventTrigger = new EventTrigger(this, "OnTrigger");
			Utils.SetupButton(this, "Actions OnTrigger", myEventTrigger.OpenPanel, false);

			myTime = Utils.SetupSliderFloatWithRange(this, "Time (sec)", 3, 0, 10, false);
			myTime.constrained = false;

			myClock = myTime.val;

			SuperController.singleton.onAtomUIDRenameHandlers += OnAtomRename;
			SimpleTriggerHandler.LoadAssets();
		}

		private void OnAtomRename(string oldid, string newid)
		{
			myEventTrigger.SyncAtomNames();
		}

		public override JSONClass GetJSON(bool includePhysical = true, bool includeAppearance = true, bool forceStore = false)
		{
			JSONClass jc = base.GetJSON(includePhysical, includeAppearance, forceStore);
			if (includePhysical || forceStore)
			{
				needsStore = true;
				jc[myEventTrigger.Name] = myEventTrigger.GetJSON(base.subScenePrefix);
			}
			return jc;
		}

		public override void LateRestoreFromJSON(JSONClass jc, bool restorePhysical = true, bool restoreAppearance = true, bool setMissingToDefault = true)
		{
			base.LateRestoreFromJSON(jc, restorePhysical, restoreAppearance, setMissingToDefault);
			Utils.AdjustSliderRange(myTime);
			myClock = myTime.val;

			if (!base.physicalLocked && restorePhysical && !IsCustomPhysicalParamLocked("trigger"))
			{
				myEventTrigger.RestoreFromJSON(jc, base.subScenePrefix, base.mergeRestore, setMissingToDefault);
			}
		}

		private void OnDestroy()
		{
			SuperController.singleton.onAtomUIDRenameHandlers -= OnAtomRename;
			myEventTrigger.Remove();
		}

		private void Update()
		{
			myEventTrigger.Update();
			
			myClock -= Time.deltaTime;
			if (myClock <= 0.0f)
			{
				myEventTrigger.Trigger();
				myClock += myTime.val;
				myClock = Mathf.Max(myClock, 0.0f);
			}
		}
    }
}
