﻿using UnityEngine;
using System.Collections.Generic;
using SimpleJSON;

namespace MacGruber
{
    public class ValueRandom : MVRScript
    {
		private FloatTrigger myFloatTrigger;

        public override void Init()
		{
			Utils.SetupInfoText(this,
				"<color=#606060><size=40><b>ValueRandom</b></size>\nLogicBrick that provides a random value between 0 and 1 when <i>Trigger</i> is called.</color>",
				200.0f, true
			);

			myFloatTrigger = new FloatTrigger(this, "OnTrigger");
			Utils.SetupButton(this, "Actions OnTrigger", myFloatTrigger.OpenPanel, false);

			Utils.SetupButton(this, "Test Trigger", Trigger, false);
			Utils.SetupAction(this, "Trigger", Trigger);

			SuperController.singleton.onAtomUIDRenameHandlers += OnAtomRename;
			SimpleTriggerHandler.LoadAssets();
		}

		private void OnAtomRename(string oldid, string newid)
		{
			myFloatTrigger.SyncAtomNames();
		}

		public override JSONClass GetJSON(bool includePhysical = true, bool includeAppearance = true, bool forceStore = false)
		{
			JSONClass jc = base.GetJSON(includePhysical, includeAppearance, forceStore);
			if (includePhysical || forceStore)
			{
				needsStore = true;
				jc[myFloatTrigger.Name] = myFloatTrigger.GetJSON(base.subScenePrefix);
			}
			return jc;
		}

		public override void LateRestoreFromJSON(JSONClass jc, bool restorePhysical = true, bool restoreAppearance = true, bool setMissingToDefault = true)
		{
			base.LateRestoreFromJSON(jc, restorePhysical, restoreAppearance, setMissingToDefault);
			if (!base.physicalLocked && restorePhysical && !IsCustomPhysicalParamLocked("trigger"))
			{
				myFloatTrigger.RestoreFromJSON(jc, base.subScenePrefix, base.mergeRestore, setMissingToDefault);
			}
		}

		private void OnDestroy()
		{
			SuperController.singleton.onAtomUIDRenameHandlers -= OnAtomRename;
			myFloatTrigger.Remove();
		}
		
		private void Update()
		{
			myFloatTrigger.Update();
		}

		private void Trigger()
		{
			float v = UnityEngine.Random.value;
			myFloatTrigger.Trigger(v);
		}
    }
}
