﻿using UnityEngine;
using System.Collections.Generic;
using SimpleJSON;

namespace MacGruber
{
    public class ValueSmoothStep : MVRScript
    {
		private JSONStorableFloat myValue;
		private FloatTrigger myFloatTrigger;

        public override void Init()
		{
			Utils.SetupInfoText(this,
				"<color=#606060><size=40><b>ValueSmoothStep</b></size>\nLogicBrick that relays a float value after applying a classic SmoothStep curve (FadeIn - FadeOut).</color>",
				200.0f, true
			);

			myFloatTrigger = new FloatTrigger(this, "OnValueSet");
			Utils.SetupButton(this, "Actions OnValueSet", myFloatTrigger.OpenPanel, false);

			myValue = Utils.SetupSliderFloat(this, "Value", 0, 0, 1, false);
			myValue.setCallbackFunction += Trigger;

			SuperController.singleton.onAtomUIDRenameHandlers += OnAtomRename;
			SimpleTriggerHandler.LoadAssets();
		}

		private void OnAtomRename(string oldid, string newid)
		{
			myFloatTrigger.SyncAtomNames();
		}

		public override JSONClass GetJSON(bool includePhysical = true, bool includeAppearance = true, bool forceStore = false)
		{
			JSONClass jc = base.GetJSON(includePhysical, includeAppearance, forceStore);
			if (includePhysical || forceStore)
			{
				needsStore = true;
				jc[myFloatTrigger.Name] = myFloatTrigger.GetJSON(base.subScenePrefix);
			}
			return jc;
		}

		public override void LateRestoreFromJSON(JSONClass jc, bool restorePhysical = true, bool restoreAppearance = true, bool setMissingToDefault = true)
		{
			base.LateRestoreFromJSON(jc, restorePhysical, restoreAppearance, setMissingToDefault);
			if (!base.physicalLocked && restorePhysical && !IsCustomPhysicalParamLocked("trigger"))
			{
				myFloatTrigger.RestoreFromJSON(jc, base.subScenePrefix, base.mergeRestore, setMissingToDefault);
			}
		}

		private void OnDestroy()
		{
			SuperController.singleton.onAtomUIDRenameHandlers -= OnAtomRename;
			myFloatTrigger.Remove();
		}
		
		private void Update()
		{
			myFloatTrigger.Update();
		}

		private void Trigger(float v)
		{
			v = Mathf.SmoothStep(0.0f, 1.0f, v);
			myFloatTrigger.Trigger(v);
		}
    }
}
